<?php

namespace Drupal\gitlab_api;

/**
 * A commit action object for \Drupal\gitlab_api\Api::createCommit.
 */
class CommitAction {

  public const TYPE_CREATE = 'create';
  public const TYPE_DELETE = 'delete';
  public const TYPE_MOVE = 'move';
  public const TYPE_UPDATE = 'update';

  public const ENCODING_TEXT = 'text';
  public const ENCODING_BASE64 = 'base64';

  protected string $type;

  protected string $path;

  protected string $content;

  protected string $encoding;

  protected ?string $previousPath;

  /**
   * @param string $type
   * @param string $path
   * @param string $content
   * @param string $encoding
   * @param string $previousPath
   *
   * @return \Drupal\gitlab_api\CommitAction
   *
   * @see \Gitlab\Api\Repositories::createCommit
   */
  public static function create(string $type, string $path, string $content, string $encoding = self::ENCODING_TEXT, string $previousPath = ''): CommitAction {
    assert(in_array($type, [self::TYPE_CREATE, self::TYPE_DELETE, self::TYPE_MOVE, self::TYPE_UPDATE]));
    assert(in_array($encoding, [self::ENCODING_TEXT, self::ENCODING_BASE64]));
    if ($type === self::TYPE_MOVE) {
      assert($previousPath !== '');
    }
    $instance = new static();
    $instance->type = $type;
    $instance->path = $path;
    $instance->content = $content;
    $instance->encoding = $encoding;
    $instance->previousPath = $previousPath;
    return $instance;
  }

  /**
   * @return array
   */
  public function toArray(): array {
    return [
      'action' => $this->type,
      'file_path' => $this->path,
      'content' => $this->content,
      'encoding' => $this->encoding,
      'previous_path' => $this->previousPath,
    ];
  }

}
